from maltego_trx.decorator_registry import TransformSetting

api_key_setting = TransformSetting(name='api_key',
                                   display_name='API Key',
                                   setting_type='string',
                                   global_setting=True)

language_setting = TransformSetting(name='language',
                                    display_name="Language",
                                    setting_type='string',
                                    default_value='en',
                                    optional=True,
                                    popup=True)



import re

import pytest

from maltego_trx.mtz import (
    create_local_server_xml,
    create_settings_xml,
    create_transform_xml,
    create_transform_set_xml,
)
from tests.test_xml import _serialize_xml


def test_create_local_server_xml(mocker, snapshot):
    mocker.patch(
        "maltego_trx.mtz.create_last_sync_timestamp",
        return_value="2022-08-10 07:52:45 UTC",
    )

    transforms = ["to_lower", "to_upper", "to_title"]

    server_xml = create_local_server_xml(transforms)
    server_xml_str = _serialize_xml(server_xml)

    assert server_xml_str == snapshot


@pytest.mark.parametrize(
    "kwargs",
    [
        {
            "working_dir": ".",
            "command": "python3",
            "params": "project.py",
            "debug": True,
        },
        {
            "working_dir": "~/project/maltego",
            "command": "venv/bin/python3",
            "params": "main.py",
            "debug": False,
        },
    ],
)
def test_create_settings_xml(kwargs, snapshot):
    settings_xml = create_settings_xml(**kwargs)
    settings_xml_str = _serialize_xml(settings_xml)

    assert settings_xml_str == snapshot


@pytest.mark.parametrize(
    "kwargs",
    [
        {
            "name": "to_lower",
            "display_name": "To Lower",
            "description": "Converts the input to lowercase",
            "input_entity": "maltego.Phrase",
            "author": "Maltego Team",
        },
        {
            "name": "to_upper",
            "display_name": "To Upper",
            "description": "Converts the input to uppercase",
            "input_entity": "maltego.Text",
            "author": "Maltego Organization",
        },
    ],
)
def test_create_transform_xml(kwargs, snapshot):
    transform_xml = create_transform_xml(**kwargs)
    transform_xml_str = _serialize_xml(transform_xml)

    assert transform_xml_str == snapshot


@pytest.mark.parametrize(
    "kwargs",
    [
        {
            "name": "text_transforms",
            "description": "Basic text transforms",
            "transforms": ["to_lower", "to_upper", "to_title"],
        },
        {
            "name": "name_transforms",
            "description": "Name Transforms",
            "transforms": ["remove_lastname", "expand_middle_name", "to_initials"],
        },
    ],
)
def test_create_transform_set_xml(kwargs, snapshot):
    transform_set_xml = create_transform_set_xml(**kwargs)
    transform_set_xml_str = _serialize_xml(transform_set_xml)

    assert transform_set_xml_str == snapshot



import sys

import transforms
from extensions import registry
from maltego_trx.handler import handle_run
from maltego_trx.registry import register_transform_classes
from maltego_trx.server import app as application

register_transform_classes(transforms)

registry.write_transforms_config(include_output_entities=True)
registry.write_settings_config()

if __name__ == '__main__':
    handle_run(__name__, sys.argv, application)



import os

from maltego_trx.oauth import MaltegoOauth

okta_ciphertoken = "pZ598ZEZ7EwpBQSOJSvCZJKkcWhtbX95K7Q0f0hwbk93O+xaUB4/NegK3r54PH1NReis/Jgt4UbGc5oCuU+R7UDM1icoDUmyCmV1U78iqjHElPdDBHlxPrl2zoXBwcU1iFbukDNy49Xghy3cwwqhmEXg/FKnYUlBQl6jdf06kE1pwfHiRgF5NrJISAD7eCmUybqiAVRDW4lbLeMGk/uSrrCpDQBxd2Em/sYKBzqO87pQRPLll23G9KNxyVinHQksGgc+dI0OPYv9EQcUs7g1JFraUKUeFjHAf/OyGGrp6WH84nGqG35afkH9xsDmySBb5DZWRjef8DzSd7oRagoGq3wKBHDh700mMKf/YkR8L9cK4l1w4yNh7EfIVlCWD5yWJUx4a1lN9CvJmFs7N9A903spVVGeP8avz50raWFb5g/4XhtIpei4ylVFi49dDeVjgb0BR0I4vuU7VDmcwFweAtSfclPb5Xfp98zI9yXnr8DB31gmzf3DIkVFIIFLxZe0gG+BSIEmC7/z0L7J+YvIqoGDq8jC0Ehe9IVXYfK1yluWAsbwj62rNAzkPVSe5l+FMQFrQRXpxZFBMXzphqRAhyYKfGNX/tocoMJoLFp8O3PBRC0kIHjTNaSX8KpftNYWreEdZx6wjk2n2eeOXM5nVeSOx4uoIifRe4NUV7VYzbGm3FSRc0k1HZIGHwn1WTvn95U3crvwFN86N3k41JU+0kzSpsLxy+Y63VfPxYTbRUET/wIOf0NTfG96QCPtVKkdVh9aTqnhaN0TpGoaZQEjUqpURxbj+8mT3g6MI4vOC9Qr6vw2wIkvzehr/yKxh6vsnfUUw8UJ1YtlZI7exPsHyFLUr2J3VvpDsaemRcn1KY/uHJZ3MMHK9GNciLA6Skw8SL38mw1tJJZ1h1bMMk1IYgBZLe2ZSXXyv7JaDPq/WJmHWDaIalmXE1cKMA56qMgYGdHxs3Pai5nwEtzIGVS1OcuMnBF6sOMMBTKSFAJmlcYo8NGwjKqe/yy1cVJZe7Ucv/jy/IS3AdE1my8cDya92jDsxifsaONQMm6YIs54SIA=$f7TU33wTTxYFtbVnwJeGGw==$c64l75qukq+980EkB1tQZKEicryt6HBB0IOjeMSRQmLOYCQpTcaXlPRJ3QdS+DfRocor7OL36wLAooRqz1IaN4QQ5+6Wx5reEXFVgKtaDX777I046DM5QU3Jf+iibSbm3mXYMw0z+OknbXyGLnc7ceSaPTJdP1LkCGQ75fLMXHCLWSpqwOKHOBhqyQGwrYlj2WxPzmOfAMaIkjKZIQWzoGjrYRzwkNCQbxykJcwD5TVuVDwAHyp84zcvW0WWoUZ+rrrooJwuJJQEdiTwLZsseqklXRNso4e5eFQwH49T9IDPHkKfVusu6rLiMNgFyc18rFR1d/BYLXBu7uzMAuvQ8wZdcOtJYx+JLJmOaPI65ymGNFTpwTHYnDBTXmpW0qX2dtEglAERw1nrdl94fwsa+I/iw3H7VIivqRF3pAchfvdNF75MIEm6XH2UXY/sZD7zjMdxxwKQiaEg2bpLd2mEtqsLc0mTq/zd03ZLEUnzXsXlp8brCfDgjoVsZAJH+ElQ6wr3dxzRMlDIxWWBYFcv8LnGVQk/Vciqtee780Yh/lgttv06kyXz81dIWjumz8TqV2eV9ZpP/YxTnNzAa91fleGNah/IKhMOV8PsGy2uOnHRiUL218p1T6+Cm9B2kmL1C/2MM3080NjJVWIfqYsyTGPbR+hURK0RB/oteG5QWPs="
private_key_path = "test_private_key.pem"


def testOktaDecrypt():
    decrypted_linkedIn_token = MaltegoOauth.decrypt_secrets(private_key_path, okta_ciphertoken)['token']
    expected_token = "eyJraWQiOiJ4ZzEyMmNZMXQ0NU5GM2l5YlUzUF9tWGluYkxEXzVrRHFwMkQ0VEl2RGd3IiwiYWxnIjoiUlMyNTYifQ.eyJ2ZXIiOjEsImp0aSI6IkFULklldnVNcXNJNDFqMURldEdPU1N2Q2VoNlI5eUxEQ0EydFBJdl9FZ3hDU3ciLCJpc3MiOiJodHRwczovL2Rldi02MTcyNDUub2t0YS5jb20vb2F1dGgyL2RlZmF1bHQiLCJhdWQiOiJhcGk6Ly9kZWZhdWx0IiwiaWF0IjoxNTg5NzgzNjU0LCJleHAiOjE1ODk3ODcyNTQsImNpZCI6IjBvYTR1YTZ1YlJIeFRDTGg4NHg2IiwidWlkIjoiMDB1NHU0ZzA4SDFqYXQwVnA0eDYiLCJzY3AiOlsib3BlbmlkIl0sInN1YiI6InRtQG1hbHRlZ28uY29tIn0.dAZrIZ9NCIqIx3fr8_0EexYCzarj8CC4CvWpVgZQCvfhtV08tGnMdWN8yuzADYvJUSzDz_meqIMMUCaVOpYZ5vzepr3LZfT-Xn00KoaaRcHGLPowphvsEPkpimvJqgnRmWw0e0VTH5Pfg9eyl3o2UUUsDofM-RkJNjxB4Uf0D6IyZaCyl0s_KhcXGZuh7hDGoR76UcKaCqRXqmqnOZs_GoMPIoDS5NHIze0MOK6sgKr8uiLikIhdh481n8PyWzPX2XZLUNcjogqX280so6Ki24VBloyxt5VgAJ1gV-e9SDj-9QSdQohQNDDSHbgiO4s1TUlhFKIm5UQWUImdhSuv1w"
    assert decrypted_linkedIn_token == expected_token


def testReadKey():
    assert os.path.exists(private_key_path)


def testCipherSplit():
    encrypted_fields = okta_ciphertoken.split("$")
    assert len(encrypted_fields) == 3


def testToken_Field_Creation():
    encrypted_fields = okta_ciphertoken.split("$")
    aes_key = MaltegoOauth._rsa_decrypt(private_key_path, encrypted_fields[2])
    token = MaltegoOauth._aes_decrypt(aes_key, encrypted_fields[0])
    token_secret = MaltegoOauth._aes_decrypt(aes_key, encrypted_fields[1])
    token_fields = {
        "token": token,
        "token_secret": token_secret
    }
    decrypted_azure_token_dict = MaltegoOauth.decrypt_secrets(private_key_path, okta_ciphertoken)
    assert token_fields == decrypted_azure_token_dict



import os.path

from maltego_trx.registry import register_transform_classes
from maltego_trx.server import app
from tests import transforms

__TESTDIR__ = os.path.dirname(__file__)


def test_request_property_mapping():
    register_transform_classes(transforms)
    app.testing = True

    with app.test_client() as test_app:
        response = make_transform_call(test_app, "/run/testrequestpropertymapping/")
        assert response.status_code == 200
        data = response.data.decode('utf8')
        assert "whois-info found" in data


def make_transform_call(test_app=None, run_endpoint=""):
    with open(os.path.join(__TESTDIR__, 'test_request.xml')) as requestMsg:
        response = test_app.post(run_endpoint, data=requestMsg.read())
    return response



import logging
import uuid
from xml.dom import minidom
from xml.etree.ElementTree import Element, SubElement

from .entities import translate_legacy_property_name, entity_property_map
from .overlays import OverlayPosition, OverlayType
from .utils import remove_invalid_xml_chars, serialize_xml, deprecated, logger

BOOKMARK_COLOR_NONE = "-1"
BOOKMARK_COLOR_BLUE = "0"
BOOKMARK_COLOR_GREEN = "1"
BOOKMARK_COLOR_YELLOW = "2"
BOOKMARK_COLOR_PURPLE = "3"
BOOKMARK_COLOR_RED = "4"
BOOKMARK_CLRS = {
    "none": BOOKMARK_COLOR_NONE,
    "blue": BOOKMARK_COLOR_BLUE,
    "green": BOOKMARK_COLOR_GREEN,
    "yellow": BOOKMARK_COLOR_YELLOW,
    "purple": BOOKMARK_COLOR_PURPLE,
    "red": BOOKMARK_COLOR_RED,
}

LINK_STYLE_NORMAL = "0"
LINK_STYLE_DASHED = "1"
LINK_STYLE_DOTTED = "2"
LINK_STYLE_DASHDOT = "3"
LINK_STYLES = {
    "normal": LINK_STYLE_NORMAL,
    "dashed": LINK_STYLE_DASHED,
    "dotted": LINK_STYLE_DOTTED,
    "dashdot": LINK_STYLE_DASHDOT,
}

UIM_FATAL = 'FatalError'
UIM_PARTIAL = 'PartialError'
UIM_INFORM = 'Inform'
UIM_DEBUG = 'Debug'
UIM_TYPES = {
    "fatal": UIM_FATAL,
    "partial": UIM_PARTIAL,
    "inform": UIM_INFORM,
    "debug": UIM_DEBUG,
}

ADD_FIELD_TEMPLATE = "<Field MatchingRule=\"%(matching)s\" Name=\"%(name)s\" DisplayName=\"%(display)s\"><![CDATA[%(value)s]]></Field>"
DISP_INFO_TEMPLATE = "<Label Name=\"%(name)s\" Type=\"text/html\"><![CDATA[' %(content)s ']]></Label>"
UIM_TEMPLATE = "<UIMessage MessageType=\"%(type)s\">%(text)s</UIMessage>"
OVERLAY_TEMPLATE = "<Overlay position=\"%(position)s\" propertyName=\"%(property_name)s\" type=\"%(type)s\"/>"


class MaltegoEntity(object):
    def __init__(self, type=None, value=None):
        if not type:
            logger.warning("Entity has no Type and will default to maltego.Phrase")
            type = 'maltego.Phrase'

        if not value:
            logger.warning("Entity has no Value and will default to ''")
            value = ''

        self.entityType = type
        self.value = value

        self.weight = 100
        self.additionalFields = []
        self.displayInformation = []
        self.iconURL = ""
        self.overlays = []

    def setType(self, type=None):
        if type:
            self.entityType = type

    def setValue(self, value=None):
        if value:
            self.value = value

    def setWeight(self, weight=None):
        if weight:
            self.weight = weight

    def addDisplayInformation(self, content=None, title='Info'):
        if content and title:
            self.displayInformation.append([title, content])

    def addProperty(self, fieldName=None, displayName=None, matchingRule='loose', value=None):
        self.additionalFields.append([fieldName, displayName, matchingRule, value])

    def setIconURL(self, url=None):
        if url:
            self.iconURL = url

    def setLinkColor(self, color):
        self.addProperty('link#maltego.link.color', 'LinkColor', '', color)

    def setLinkStyle(self, style):
        self.addProperty('link#maltego.link.style', 'LinkStyle', '', style)

    def setLinkThickness(self, thick):
        self.addProperty('link#maltego.link.thickness', 'Thickness', '', str(thick))

    def setLinkLabel(self, label):
        self.addProperty('link#maltego.link.label', 'Label', '', label)

    def reverseLink(self):
        self.addProperty('link#maltego.link.direction', 'link#maltego.link.direction', 'loose', 'output-to-input')

    def addCustomLinkProperty(self, fieldName=None, displayName=None, value=None):
        if fieldName:
            self.addProperty('link#' + fieldName, displayName, '', value)

    def setBookmark(self, bookmark):
        self.addProperty('bookmark#', 'Bookmark', '', bookmark)

    def setNote(self, note):
        self.addProperty('notes#', 'Notes', '', note)

    def addOverlay(
            self, propertyName, position: OverlayPosition, overlayType: OverlayType
    ):
        self.overlays.append([propertyName, position.value, overlayType.value])

    @deprecated()
    def add_field_to_xml(self, additional_field):
        name, display, matching, value = additional_field
        matching = "strict" if matching.lower().strip() == "strict" else "loose"

        return ADD_FIELD_TEMPLATE % {
            "matching": matching,
            "name": name,
            "display": display,
            "value": remove_invalid_xml_chars(value),
        }

    @deprecated()
    def disp_info_to_xml(self, disp_info):
        name, content = disp_info

        return DISP_INFO_TEMPLATE % {
            "content": remove_invalid_xml_chars(content),
            "name": name,
        }

    def build_xml(self) -> Element:
        # defaults are set to allow for backwards compatibility with old serializer

        entity_xml = Element('Entity', attrib={'Type': self.entityType})

        value_xml = SubElement(entity_xml, 'Value')
        value_xml.text = str(self.value)

        if not self.weight:
            logger.warning("Entity has no Weight and will default to 100")
            self.weight = 100

        weight_xml = SubElement(entity_xml, 'Weight')
        weight_xml.text = str(self.weight)

        if self.displayInformation:
            display_infos_xml = SubElement(entity_xml, 'DisplayInformation')
            for display_info in self.displayInformation:
                title, content = display_info

                if not title:
                    logger.warning(f"Display information is missing title and will default to 'Info': "
                                   f"title={title}")
                    title = 'Info'

                if not content:
                    logging.warning(f"Display information is missing content: "
                                    f"content={content}")
                    content = ""

                display_info_xml = SubElement(display_infos_xml, 'Label', attrib={'Name': title, 'Type': "text/html"})
                # for some reason, the client accepts escaped html and renders it correctly, so we don't need CDATA
                display_info_xml.text = str(content)

        if self.additionalFields:
            properties_xml = SubElement(entity_xml, 'AdditionalFields')
            for prop in self.additionalFields:
                field_name, display_name, matching_rule, value = prop

                if not field_name:
                    logger.error(f"No property name specified. "
                                 f"field_name={field_name}, display_name={display_name}, "
                                 f"matching_rule={matching_rule}, value={value}")

                # the client will still use the entity definitions display value
                # if there is none, it would use the empty string, so we use the title as a backup
                display_name = display_name or field_name

                matching_rule = "strict" if matching_rule == "strict" else "loose"

                field_xml = SubElement(properties_xml, 'Field',
                                       attrib={'Name': str(field_name),
                                               'DisplayName': str(display_name),
                                               'MatchingRule': matching_rule})
                field_xml.text = str(value or "")

        if self.overlays:
            overlays_xml = SubElement(entity_xml, 'Overlays')
            for overlay in self.overlays:
                property_name, position, overlay_type = overlay

                if not all((property_name, position, overlay_type)):
                    logging.warning(f"Overlay is missing a property name, position or type: "
                                    f"property_name={property_name}, position={position}, overlay_type={overlay_type}")

                SubElement(overlays_xml, 'Overlay',
                           attrib={'propertyName': str(property_name),
                                   'position': position,
                                   'type': overlay_type})

        if self.iconURL:
            icon_xml = SubElement(entity_xml, 'IconURL')
            icon_xml.text = self.iconURL

        return entity_xml

    def returnEntity(self):
        return serialize_xml(self.build_xml())


class MaltegoTransform(object):
    def __init__(self):
        self.entities = []
        self.exceptions = []
        self.UIMessages = []

    def addEntity(self, type=None, value=None):
        entity = MaltegoEntity(type, value)
        self.entities.append(entity)
        return entity

    def addUIMessage(self, message, messageType="Inform"):
        self.UIMessages.append([messageType, message])

    def addException(self, exceptionString):
        self.exceptions.append(exceptionString)

    def build_exceptions_xml(self):
        message_xml = Element('MaltegoMessage')
        exceptions_message_xml = SubElement(message_xml, 'MaltegoTransformExceptionMessage')
        exceptions_xml = SubElement(exceptions_message_xml, 'Exceptions')

        for exception in self.exceptions:
            exception_xml = SubElement(exceptions_xml, 'Exception')
            exception_xml.text = exception

        return message_xml

    def throwExceptions(self):
        return serialize_xml(self.build_exceptions_xml())

    def build_xml(self) -> Element:
        message_xml = Element('MaltegoMessage')
        response_xml = SubElement(message_xml, 'MaltegoTransformResponseMessage')

        entities_xml = SubElement(response_xml, 'Entities')
        for entity in self.entities:
            entities_xml.append(entity.build_xml())

        ui_messages_xml = SubElement(response_xml, 'UIMessages')
        for ui_message in self.UIMessages:
            message_type, message_content = ui_message
            if not all((message_type, message_content)):
                message_type = message_type or UIM_INFORM
                logging.warning(f"UIMessage is missing a message type or content: "
                                f"message_type={message_type}, message_content={message_content}")

            ui_message_xml = SubElement(ui_messages_xml, 'UIMessage', attrib={'MessageType': message_type})
            ui_message_xml.text = str(message_content)

        return message_xml

    def returnOutput(self):
        return serialize_xml(self.build_xml())


class MaltegoMsg:
    @staticmethod
    def _get_text(element):
        for child in element.childNodes:
            if child.nodeType == child.TEXT_NODE:
                return child.data

    @staticmethod
    def _get_int(element, tag_name, attr_name=None):
        try:
            element = element.getElementsByTagName(tag_name)[0]
            value = MaltegoMsg._get_text(element) if not attr_name else element.getAttribute(attr_name)
            return int(value)
        except ValueError:
            print("Error: Unable to convert XML value for '%s' to an integer." % tag_name)
            return 0

    def __init__(self, MaltegoXML="", LocalArgs=[]):
        self.TransformSettings = {}
        self.Properties = {}

        if MaltegoXML:
            maltego_msg = minidom.parseString(MaltegoXML)
            entities = maltego_msg.getElementsByTagName("Entity")
            entity = entities[0]

            self.Value = self._get_text(entity.getElementsByTagName("Value")[0])
            self.Type = entity.attributes["Type"].value

            self.Weight = self._get_int(entity, "Weight")
            self.Slider = self._get_int(maltego_msg, "Limits", attr_name="SoftLimit")
            self.Genealogy = []
            genealogy_tag = maltego_msg.getElementsByTagName("Genealogy")
            genealogy_types = genealogy_tag[0].getElementsByTagName("Type") if genealogy_tag else []
            for genealogy_type_tag in genealogy_types:
                entity_type_name = genealogy_type_tag.getAttribute("Name")
                entity_type_old_name = genealogy_type_tag.getAttribute("OldName")
                entity_type = {"Name": entity_type_name,
                               "OldName": entity_type_old_name if entity_type_old_name else None}
                self.Genealogy.append(entity_type)

            # Additional Fields
            additional_fields_tag = entity.getElementsByTagName("AdditionalFields")
            additional_fields = additional_fields_tag[0].getElementsByTagName("Field") if additional_fields_tag else []
            for field in additional_fields:
                name = field.getAttribute("Name")
                value = self._get_text(field)
                self.Properties[name] = value
                for entity_type in self.Genealogy:
                    v3_property_name = translate_legacy_property_name(entity_type["Name"], name)
                    if v3_property_name is not None:
                        self.Properties[v3_property_name] = value

            # Transform Settings
            settings_tag = maltego_msg.getElementsByTagName("TransformFields")
            settings = settings_tag[0].getElementsByTagName("Field") if settings_tag else []
            for setting in settings:
                name = setting.getAttribute("Name")
                value = self._get_text(setting)
                self.TransformSettings[name] = value

        elif LocalArgs:
            self.Value = LocalArgs[0]
            self.Type = "local.Unknown"
            self.Genealogy = None

            self.Weight = 100
            self.Slider = 100

            if len(LocalArgs) > 1:
                hash_rnd = uuid.uuid4().hex.upper()[0:7]
                equals_rnd = uuid.uuid4().hex.upper()[0:7]
                bslash_rnd = uuid.uuid4().hex.upper()[0:7]
                text = LocalArgs[1] \
                    .replace("\\\\", bslash_rnd) \
                    .replace("\\#", hash_rnd) \
                    .replace("\\=", equals_rnd)

                self.buildProperties(text.split("#"), hash_rnd, equals_rnd, bslash_rnd)

    def clearLegacyProperties(self):
        to_clear = set()
        for entity_type in self.Genealogy or []:
            for prop_name in entity_property_map.get(entity_type["Name"], []):
                to_clear.add(prop_name)

        for field_name in to_clear:
            if field_name in self.Properties:
                del self.Properties[field_name]

    def buildProperties(self, key_value_array, hash_rnd, equals_rnd, bslash_rnd):
        self.Properties = {}
        for property_section in key_value_array:
            name, value = property_section.split("=", 2)
            self.Properties[name] = value \
                .replace(hash_rnd, "#") \
                .replace(equals_rnd, "=") \
                .replace(bslash_rnd, "\\")

    def getProperty(self, key):
        return self.Properties.get(key)

    def getTransformSetting(self, key):
        return self.TransformSettings.get(key)



import os
import shutil
import sys

import maltego_trx

"""
Receive commands run to start a new project.
"""


def execute_from_command_line():
    args = sys.argv[1:]

    if args[0].lower() == "start":
        run_start(args[1:])
    elif args[0].lower() == "init":
        run_start([], create_dir=False)
    else:
        print("Command not recognised. Available commands are: \n 'start <project_name>' \n 'init'")


def run_start(args, create_dir: bool = True):
    if create_dir:
        project = args[0]
        project_dir = os.path.join(os.getcwd(), project)
        os.makedirs(project_dir)
    else:
        project_dir = os.getcwd()
        project = os.path.basename(project_dir)

    try:
        template_dir_path = os.path.join(maltego_trx.__path__[0], "template_dir")
        copytree(template_dir_path, project_dir)
        print("Successfully created a new project in the '%s' folder." % project)
    except FileExistsError:
        print("ERROR: '%s' already exists" % project_dir)
    except OSError as e:
        print("ERROR: %s" % str(e))


def copytree(src, dst, symlinks=False, ignore=None):
    for item in os.listdir(src):
        if "__pycache__" not in item and ".pyc" not in item:
            s = os.path.join(src, item)
            d = os.path.join(dst, item)
            if os.path.isdir(s):
                shutil.copytree(s, d, symlinks, ignore)
            else:
                shutil.copy2(s, d)



import logging
import os
import zipfile
from collections import defaultdict
from dataclasses import dataclass, field
from itertools import chain
from typing import List, Optional, Dict, Iterable, Tuple

from maltego_trx.mtz import (
    create_local_server_xml,
    create_settings_xml,
    create_transform_xml,
    create_transform_set_xml,
)
from maltego_trx.utils import (
    filter_unique,
    pascal_case_to_title,
    escape_csv_fields,
    export_as_csv,
    serialize_bool,
    name_to_path,
    serialize_xml,
)

LEGACY_TRANSFORMS_CSV_HEADER = (
    "Owner,Author,Disclaimer,Description,Version,"
    "Name,UIName,URL,entityName,"
    "oAuthSettingId,transformSettingIDs,seedIDs"
)

TRANSFORMS_CSV_HEADER = (
    "Owner,Author,Disclaimer,Description,Version,"
    "Name,UIName,URL,entityName,"
    "oAuthSettingId,transformSettingIDs,seedIDs,outputEntities"
)

SETTINGS_CSV_HEADER = "Name,Type,Display,DefaultValue,Optional,Popup"


@dataclass(frozen=True)
class TransformSet:
    name: str
    description: str


@dataclass()
class TransformMeta:
    class_name: str
    display_name: str
    input_entity: str
    description: str
    output_entities: List[str]
    disclaimer: str
    transform_set: TransformSet


@dataclass(frozen=True)
class TransformSetting:
    name: str
    display_name: str
    setting_type: str  # Literal['string', 'boolean', 'date', 'datetime', 'daterange', 'url', 'double', 'int']

    default_value: Optional[str] = ""
    optional: bool = False
    popup: bool = False
    global_setting: bool = False

    @property
    def id(self) -> str:
        """this setting's full id for reference"""
        if self.global_setting:
            return "global#" + self.name
        return self.name


@dataclass(eq=False)
class TransformRegistry:
    owner: str
    author: str

    host_url: str
    seed_ids: List[str]

    version: str = "0.1"
    display_name_suffix: str = ""

    global_settings: List[TransformSetting] = field(default_factory=list)
    oauth_settings_id: Optional[str] = ""

    transform_metas: Dict[str, TransformMeta] = field(init=False, default_factory=dict)
    transform_settings: Dict[str, List[TransformSetting]] = field(
        init=False, default_factory=dict
    )
    transform_sets: Dict[TransformSet, List[str]] = field(
        init=False, default_factory=lambda: defaultdict(list)
    )

    def register_transform(
        self,
        display_name: str,
        input_entity: str,
        description: str,
        settings: List[TransformSetting] = None,
        output_entities: List[str] = None,
        disclaimer: str = "",
        transform_set: TransformSet = None,
    ):
        """This method can be used as a decorator on transform classes. The data will be used to fill out csv config
        files to be imported into a TDS.
        """

        def decorated(transform_callable: object):
            cleaned_transform_name = name_to_path(transform_callable.__name__)
            display = display_name or pascal_case_to_title(transform_callable.__name__)

            meta = TransformMeta(
                cleaned_transform_name,
                display,
                input_entity,
                description,
                output_entities or [],
                disclaimer,
                transform_set=transform_set,
            )
            self.transform_metas[cleaned_transform_name] = meta

            if settings:
                self.transform_settings[cleaned_transform_name] = settings

            if transform_set:
                self.transform_sets[transform_set].append(cleaned_transform_name)

            return transform_callable

        return decorated

    def _create_transforms_config(self, include_output_entities: bool = True) -> Iterable[str]:
        global_settings_full_names = [gs.id for gs in self.global_settings]

        for transform_name, transform_meta in self.transform_metas.items():
            meta_settings = [
                setting.id
                for setting in self.transform_settings.get(transform_name, [])
            ]

            transform_row = [
                self.owner,
                self.author,
                transform_meta.disclaimer,
                transform_meta.description,
                self.version,
                transform_name,
                transform_meta.display_name + self.display_name_suffix,
                os.path.join(self.host_url, "run", transform_name),
                transform_meta.input_entity,
                ";".join(self.oauth_settings_id),
                # combine global and transform scoped settings
                ";".join(chain(meta_settings, global_settings_full_names)),
                ";".join(self.seed_ids)
            ]

            if include_output_entities:
                transform_row.append(";".join(transform_meta.output_entities))

            escaped_fields = escape_csv_fields(*transform_row)
            yield ",".join(escaped_fields)

    def write_transforms_config(
        self, config_path: str = "./transforms.csv", csv_line_limit: int = 100, include_output_entities: bool = True
    ):
        """Exports the collected transform metadata as a csv-file to config_path"""

        csv_lines = self._create_transforms_config(include_output_entities=include_output_entities)

        if not include_output_entities:
            export_as_csv(
                LEGACY_TRANSFORMS_CSV_HEADER, tuple(csv_lines), config_path, csv_line_limit
            )
        else:
            export_as_csv(
                TRANSFORMS_CSV_HEADER, tuple(csv_lines), config_path, csv_line_limit
            )

    def _create_settings_config(self) -> Iterable[str]:
        chained_settings = chain(
            self.global_settings, *list(self.transform_settings.values())
        )
        unique_settings: Iterable[TransformSetting] = filter_unique(
            lambda s: s.name, chained_settings
        )

        for setting in unique_settings:
            setting_row = [
                setting.id,
                setting.setting_type,
                setting.display_name,
                setting.default_value or "",
                serialize_bool(setting.optional, "True", "False"),
                serialize_bool(setting.popup, "Yes", "No"),
            ]

            escaped_fields = escape_csv_fields(*setting_row)
            yield ",".join(escaped_fields)

    def write_settings_config(
        self, config_path: str = "./settings.csv", csv_line_limit: int = 100
    ):
        """Exports the collected settings metadata as a csv-file to config_path"""

        csv_lines = self._create_settings_config()

        export_as_csv(
            SETTINGS_CSV_HEADER, tuple(csv_lines), config_path, csv_line_limit
        )

    def _create_local_mtz(
        self,
        working_dir: str = ".",
        command: str = "python3",
        params: str = "project.py",
        debug: bool = True,
    ) -> Iterable[Tuple[str, str]]:
        working_dir = os.path.abspath(working_dir)
        if self.global_settings:
            logging.warning(
                f"Settings are not supported with local transforms. "
                f"Global settings are: {', '.join(map(lambda s: s.name, self.global_settings))}"
            )

        """Creates an .mtz for bulk importing local transforms"""
        server_xml = create_local_server_xml(self.transform_metas.keys())

        server_xml_str = serialize_xml(server_xml)
        yield "Servers/Local.tas", server_xml_str

        for name, meta in self.transform_metas.items():
            settings_xml = create_settings_xml(
                working_dir, command, f"{params} local {name}", debug
            )
            settings_xml_str = serialize_xml(settings_xml)

            tx_settings = self.transform_settings.get(name)
            if tx_settings:
                logging.warning(
                    "Settings are not supported with local transforms. "
                    f"Transform '{meta.display_name}' has: {', '.join(map(lambda s: s.name, tx_settings))}"
                )

            xml = create_transform_xml(
                name,
                meta.display_name,
                meta.description,
                meta.input_entity,
                self.author,
            )

            xml_str = serialize_xml(xml)

            yield f"TransformRepositories/Local/{name}.transform", xml_str
            yield f"TransformRepositories/Local/{name}.transformsettings", settings_xml_str

        for transform_set, transforms in self.transform_sets.items():
            set_xml = create_transform_set_xml(
                transform_set.name, transform_set.description, transforms
            )

            set_xml_str = serialize_xml(set_xml)

            yield f"TransformSets/{transform_set.name}.set", set_xml_str

    def write_local_mtz(
        self,
        mtz_path: str = "./local.mtz",
        working_dir: str = ".",
        command: str = "python3",
        params: str = "project.py",
        debug: bool = True,
    ):

        with zipfile.ZipFile(mtz_path, "w") as mtz:
            for path, content in self._create_local_mtz(
                working_dir, command, params, debug
            ):
                mtz.writestr(path, content)



import logging

from flask import Flask, request

from .maltego import MaltegoMsg, MaltegoTransform
from .registry import mapping

log = logging.getLogger("maltego.server")
logging.basicConfig(level=logging.DEBUG)

URL_TEMPLATE = '/run/<transform_name>/'
URL_TEMPLATE_NO_SLASH = '/run/<transform_name>'


def get_exception_message(msg="An exception occurred with the transform. Check the logs for more details."):
    transform_run = MaltegoTransform()
    transform_run.addUIMessage(msg, "PartialError")

    return transform_run.returnOutput()


def print_transforms():
    print("= Transform Server URLs =")
    for path in mapping:
        print(URL_TEMPLATE.replace("<transform_name>", path) + ": " + mapping[path].__name__)
    print("\n")

    print("= Local Transform Names =")
    for path in mapping:
        print(path + ": " + mapping[path].__name__)
    print("\n")


def run_transform(transform_name, client_msg):
    transform_method = mapping[transform_name]
    try:
        if hasattr(transform_method, "run_transform"):
            return transform_method.run_transform(client_msg), 200  # Transform class
        else:
            return transform_method(client_msg), 200  # Transform method
    except Exception as e:
        log.error("An exception occurred while executing your transform code.")
        log.error(e, exc_info=True)
        return get_exception_message(), 200


app = Flask(__name__)
application = app  # application variable for usage with apache mod wsgi


def transform_runner(transform_name):
    transform_name = transform_name.lower()
    if transform_name in mapping:
        if request.method == 'POST':
            client_msg = MaltegoMsg(request.data)
            return run_transform(transform_name, client_msg)
        else:
            return "Transform found with name '%s', you will need to send a POST request to run it." % transform_name, 200
    else:
        log.info("No transform found with the name '%s'." % transform_name)
        log.info("Available transforms are:\n %s" % str(list(mapping.keys())))
        return "No transform found with the name '%s'." % transform_name, 404


# Add the route with and without the slash, since POSTs can't be redirected
app.route(URL_TEMPLATE_NO_SLASH, methods=['GET', 'POST'])(transform_runner)
app.route(URL_TEMPLATE, methods=['GET', 'POST'])(transform_runner)


@app.route('/', methods=['GET', 'POST'])
def index():
    return "You have reached a Maltego Transform Server.", 200



import datetime
from typing import Iterable, List
from xml.etree.ElementTree import Element, SubElement


def create_last_sync_timestamp(timestamp: datetime.datetime = None) -> str:
    timestamp = timestamp or datetime.datetime.utcnow()
    return timestamp.strftime("%Y-%m-%d %H:%M:%S UTC")


def create_local_server_xml(transform_names: Iterable[str]) -> Element:
    server_xml = Element("MaltegoServer", attrib={"name": "Local",
                                                  "enabled": "true",
                                                  "description": "Local transforms hosted on this machine",
                                                  "url": "http://localhost",
                                                  })
    last_sync_xml = SubElement(server_xml, "LastSync")
    last_sync_xml.text = create_last_sync_timestamp()

    SubElement(server_xml, "Protocol", attrib={"version": "0.0"})
    SubElement(server_xml, "Authentication", attrib={"type": "none"})

    transforms_xml = SubElement(server_xml, "Transforms")
    for name in transform_names:
        SubElement(transforms_xml, "Transform", attrib={"name": name})

    SubElement(server_xml, "Seeds")

    return server_xml


def create_settings_xml(working_dir: str, command: str, params: str, debug: bool) -> Element:
    settings_xml = Element("TransformSettings",
                           attrib={"enabled": "true",
                                   "disclaimerAccepted": "false",
                                   "showHelp": "true",
                                   "runWithAll": "true",
                                   "favorite": "false",
                                   })
    properties_xml = SubElement(settings_xml, "Properties")

    command_xml = SubElement(properties_xml, "Property",
                             attrib={"name": "transform.local.command",
                                     "type": "string",
                                     "popup": "false",
                                     })
    command_xml.text = command

    parameters_xml = SubElement(properties_xml, "Property",
                                attrib={
                                    "name": "transform.local.parameters",
                                    "type": "string",
                                    "popup": "false",
                                })
    parameters_xml.text = params

    working_directory_xml = SubElement(properties_xml, "Property",
                                       attrib={
                                           "name": "transform.local.working-directory",
                                           "type": "string",
                                           "popup": "false",
                                       })
    working_directory_xml.text = working_dir

    debug_xml = SubElement(properties_xml, "Property",
                           attrib={
                               "name": "transform.local.debug",
                               "type": "boolean",
                               "popup": "false",
                           })
    debug_xml.text = "true" if debug else "false"

    return settings_xml


def create_transform_xml(name: str, display_name: str, description: str, input_entity: str, author: str) -> Element:
    transform_xml = Element("MaltegoTransform",
                            attrib={"name": name,
                                    "displayName": display_name,
                                    "abstract": "false",
                                    "template": "false",
                                    "visibility": "public",
                                    "description": description,
                                    "author": author,
                                    "requireDisplayInfo": "false",
                                    })

    adapter_xml = SubElement(transform_xml, "TransformAdapter")
    adapter_xml.text = "com.paterva.maltego.transform.protocol.v2api.LocalTransformAdapterV2"

    properties_xml = SubElement(transform_xml, "Properties")
    fields_xml = SubElement(properties_xml, "Fields")

    SubElement(fields_xml, "Property",
               attrib={"name": "transform.local.command",
                       "type": "string",
                       "nullable": "false",
                       "hidden": "false",
                       "readonly": "false",
                       "description": "The command to execute for this transform",
                       "popup": "false",
                       "abstract": "false",
                       "visibility": "public",
                       "auth": "false",
                       "displayName": "Command line",
                       })
    SubElement(fields_xml, "Property",
               attrib={"name": "transform.local.parameters",
                       "type": "string",
                       "nullable": "true",
                       "hidden": "false",
                       "readonly": "false",
                       "description": "The parameters to pass to the transform command",
                       "popup": "false",
                       "abstract": "false",
                       "visibility": "public",
                       "auth": "false",
                       "displayName": "Command parameters",
                       })
    SubElement(fields_xml, "Property",
               attrib={"name": "transform.local.working-directory",
                       "type": "string",
                       "nullable": "true",
                       "hidden": "false",
                       "readonly": "false",
                       "description": "The working directory used when invoking the executable",
                       "popup": "false",
                       "abstract": "false",
                       "visibility": "public",
                       "auth": "false",
                       "displayName": "Working directory",

                       })
    SubElement(fields_xml, "Property",
               attrib={
                   "name": "transform.local.debug",
                   "type": "boolean",
                   "nullable": "true",
                   "hidden": "false",
                   "readonly": "false",
                   "description": "When this is set, the transform's text output will be "
                                  "printed to the output window",
                   "popup": "false",
                   "abstract": "false",
                   "visibility": "public",
                   "auth": "false",
                   "displayName": "Show debug info"
               })

    input_constraints_xml = SubElement(transform_xml, "InputConstraints")
    SubElement(input_constraints_xml, "Entity",
               attrib={"type": input_entity,
                       "min": "1", "max": "1"})

    SubElement(transform_xml, "OutputEntities")
    SubElement(transform_xml, "defaultSets")

    stealth_level = SubElement(transform_xml, "StealthLevel")
    stealth_level.text = "0"

    return transform_xml


def create_transform_set_xml(name: str, description: str, transforms: List[str]) -> Element:
    set_xml = Element("TransformSet",
                      attrib={
                          "name": name,
                          "description": description
                      })

    transforms_xml = SubElement(set_xml, "Transforms")
    for transform in transforms:
        SubElement(transforms_xml,
                   "Transform",
                   attrib={
                       "name": transform
                   })

    return set_xml



from xml.etree.ElementTree import Element

from maltego_trx.maltego import MaltegoTransform
from maltego_trx.overlays import OverlayType, OverlayPosition
from maltego_trx.utils import serialize_xml


def _serialize_xml(xml: Element) -> str:
    # we need to use canonical XML (python3.8+), because python3.6 and python3.7 sort the attrs alphabetically
    return serialize_xml(xml, indent=False, canonicalize=True, short_empty_elements=False)


def test_entity_with_value(snapshot):
    response = MaltegoTransform()
    response.addEntity("maltego.Phrase", "Hello Spencer!")
    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_entity_with_none_value(caplog, snapshot):
    response = MaltegoTransform()
    response.addEntity("maltego.Phrase", None)
    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot

    captured = caplog.messages
    assert captured == snapshot


def test_entity_with_none_type_is_phrase(caplog, snapshot):
    response = MaltegoTransform()
    response.addEntity(None, "Value")
    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot

    captured = caplog.messages
    assert captured == snapshot


def test_entity_property_with_no_field_name_gets_skipped(caplog, snapshot):
    response = MaltegoTransform()
    entity = response.addEntity("maltego.Phrase", "Hello Spencer!")
    entity.addProperty(fieldName=None, displayName="displayNameTest", value="valueTest",
                       matchingRule="loose")

    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot

    captured = caplog.messages
    assert captured == snapshot


def test_entity_property_with_no_display_name_gets_field_name(snapshot):
    response = MaltegoTransform()
    entity = response.addEntity("maltego.Phrase", "Hello Spencer!")
    entity.addProperty(fieldName="fieldNameTest", displayName=None, value="valueTest",
                       matchingRule="loose")

    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_entity_property_with_no_matching_rule_gets_loose_matching_rule(snapshot):
    response = MaltegoTransform()
    entity = response.addEntity("maltego.Phrase", "Hello Spencer!")
    entity.addProperty(fieldName="fieldNameTest", displayName="displayNameTest", matchingRule=None, value="valueTest")

    response_xml = response.build_xml()
    assert _serialize_xml(response_xml) == snapshot


def test_entity_property_with_no_value_gets_empty_value(caplog, snapshot):
    response = MaltegoTransform()
    entity = response.addEntity("maltego.Phrase", "Hello Spencer!")
    entity.addProperty(fieldName="fieldNameTest", displayName="displayNameTest", value=None, matchingRule="loose")

    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_entity_with_properties(snapshot):
    response = MaltegoTransform()
    entity = response.addEntity("maltego.Phrase", "Hello Spencer!")
    entity.addProperty(fieldName="fieldNameTest", displayName=None, value="valueTest",
                       matchingRule="loose")
    entity.addProperty(fieldName="fieldNameTest2", displayName="displayNameTest2", value="valueTest2",
                       matchingRule="strict")

    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_entity_with_display_information(snapshot):
    response = MaltegoTransform()
    entity = response.addEntity("maltego.Phrase", "Hello Spencer!")
    entity.addDisplayInformation(title="Display Info Title", content="<p>Display Info Content</p>")

    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_entity_with_icon(snapshot):
    response = MaltegoTransform()
    entity = response.addEntity("maltego.Phrase", "Hello Spencer!")
    entity.setIconURL(url="https://www.maltego.com/img/maltego-logo/maltego-horizontal.png")

    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_entity_with_overlays(snapshot):
    response = MaltegoTransform()
    entity = response.addEntity("maltego.Phrase", "Hello Spencer!")
    entity.addOverlay('#45e06f', OverlayPosition.NORTH_WEST, OverlayType.COLOUR)

    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_response_with_ui_message(snapshot):
    response = MaltegoTransform()
    response.addUIMessage("Test Message", messageType="inform")

    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_response_with_exception(snapshot):
    response = MaltegoTransform()
    response.addException("Test Exception")

    response_xml = response.build_exceptions_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_exception_message(snapshot):
    response = MaltegoTransform()
    response.addUIMessage("Test Exception", "PartialError")
    response_xml = response.build_xml()

    assert _serialize_xml(response_xml) == snapshot


def test_all_null_values(caplog, snapshot):
    response = MaltegoTransform()
    entity = response.addEntity(None, None)
    entity.addProperty(fieldName=None, displayName=None, value=None,
                       matchingRule=None)

    entity.addDisplayInformation(title=None, content=None)
    entity.setIconURL(url=None)

    entity.addOverlay(propertyName=None, position=OverlayPosition.NORTH_WEST, overlayType=OverlayType.COLOUR)

    entity.setLinkLabel(None)
    entity.setLinkThickness(None)
    entity.setLinkColor(None)
    entity.setLinkStyle(None)
    entity.setType(None)
    entity.setWeight(None)
    entity.addCustomLinkProperty(None, None, None)
    entity.setNote(None)
    entity.setValue(None)

    response.addUIMessage(None, messageType=None)
    response.addException(None)

    response_xml = response.build_xml()
    assert _serialize_xml(response_xml) == snapshot

    captured = caplog.messages
    assert captured == snapshot



"""Maltego OAuth Crypto Helper"""
import base64

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization, padding as primitives_padding
from cryptography.hazmat.primitives.asymmetric import padding as asymmetric_padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from requests.auth import AuthBase


class MaltegoOauth:
    """
    A Crypto Helper for Maltego OAuth Secrets received from the Transform Distribution Server
    The TDS will send back an encrypted combination of the following :
    1. Token
    2. Token Secret
    3. Refresh Token
    4. Expires In

    Contains Methods:
        1. decrypt_secrets(private_key_path="pem file", ciphertext="request.getTransformSetting('name from TDS')")
    """

    @staticmethod
    def _rsa_decrypt(private_key_path=None, ciphertext=None, password=None):
        """
        RSA Decryption function, returns decrypted plaintext in b64 encoding
        """
        ciphertext = base64.b64decode(ciphertext)

        with open(private_key_path, "rb") as key_file:
            private_key = serialization.load_pem_private_key(key_file.read(),
                                                             password,
                                                             backend=None)
            plaintext = private_key.decrypt(ciphertext, asymmetric_padding.PKCS1v15())

        return plaintext

    @staticmethod
    def _aes_decrypt(key=None, ciphertext=None):
        """
        AES Decryption function, returns decrypted plaintext value
        """
        key = base64.b64decode(key)
        cipher = Cipher(algorithms.AES(key), modes.ECB(), backend=default_backend())
        decryptor = cipher.decryptor()
        ciphertext = base64.b64decode(ciphertext)
        padded_b64_plaintext = decryptor.update(ciphertext) + decryptor.finalize()
        unpadder = primitives_padding.PKCS7(128).unpadder()
        plaintext = (unpadder.update(padded_b64_plaintext) + unpadder.finalize()).decode('utf8')
        return plaintext

    @classmethod
    def decrypt_secrets(cls, private_key_path=None, encoded_ciphertext=None):
        """
        The TDS will send back an encrypted combination of the following :
        1. Token
        2. Token Secret
        3. Refresh Token
        4. Expires In

        This function decodes the combinations and decrypts as required and returns a dictionary with the following keys
                {"token":"",
                "token_secret": "",
                "refresh_token": "",
                "expires_in": ""}
        """

        encrypted_fields = encoded_ciphertext.split("$")

        if len(encrypted_fields) == 1:
            token = cls._rsa_decrypt(private_key_path, encrypted_fields[0])
            token_fields = {
                "token": token
            }

        elif len(encrypted_fields) == 2:
            token = cls._rsa_decrypt(private_key_path, encrypted_fields[0])
            token_secret = cls._rsa_decrypt(private_key_path, encrypted_fields[1])
            token_fields = {
                "token": token,
                "token_secret": token_secret
            }

        elif len(encrypted_fields) == 3:
            aes_key = cls._rsa_decrypt(private_key_path, encrypted_fields[2])
            token = cls._aes_decrypt(aes_key, encrypted_fields[0])
            token_secret = cls._aes_decrypt(aes_key, encrypted_fields[1])
            token_fields = {
                "token": token,
                "token_secret": token_secret
            }
        elif len(encrypted_fields) == 4:
            token = cls._rsa_decrypt(private_key_path, encrypted_fields[0])
            token_secret = cls._rsa_decrypt(private_key_path, encrypted_fields[1])
            refresh_token = cls._rsa_decrypt(private_key_path, encrypted_fields[2])
            expires_in = cls._rsa_decrypt(private_key_path, encrypted_fields[3])
            token_fields = {
                "token": token,
                "token_secret": token_secret,
                "refresh_token": refresh_token,
                "expires_in": expires_in
            }
        elif len(encrypted_fields) == 5:
            aes_key = cls._rsa_decrypt(private_key_path, encrypted_fields[4])
            token = cls._aes_decrypt(aes_key, encrypted_fields[0])
            token_secret = cls._aes_decrypt(aes_key, encrypted_fields[1])
            refresh_token = cls._aes_decrypt(aes_key, encrypted_fields[2])
            expires_in = cls._aes_decrypt(aes_key, encrypted_fields[3])
            token_fields = {
                "token": token,
                "token_secret": token_secret,
                "refresh_token": refresh_token,
                "expires_in": expires_in
            }
        else:
            token_fields = {
                "token": "",
                "token_secret": "",
                "refresh_token": "",
                "expires_in": ""
            }

        return token_fields


class OAuth2BearerToken(AuthBase):
    """Implements OAuth2 Bearer access token authentication.

    Pass this object via the `auth` parameter to a request or a
    session object in order to authenticate your requests.

    Example usage, once you have the `access_token`:

    class GreetPerson(DiscoverableTransform):

        @classmethod
        def create_entities(cls, request, response):
            person_name = request.Value

            private_key_path = "private_key.pem"

            encrypted_secrets = request.getTransformSetting('maltego.web.api.key.linkedin')

            token_fields = MaltegoCrypto.decrypt_secrets(private_key_path,encrypted_secrets)

            api_url = ("https://api.linkedin.com/v2/emailAddress?q=members&projection=(elements*(handle~))")
            auth = OAuth2BearerToken(token_fields['token'])
            result = requests.get(api_url,auth=auth)

            response.addEntity(Phrase, result.text)
    """

    def __init__(self, access_token):
        self.access_token = access_token

    def __call__(self, request):
        request.headers['Authorization'] = 'Bearer {}'.format(
            self.access_token
        )
        return request



import logging
import math
import re
import sys
import warnings
from typing import TypeVar, Callable, Hashable, Iterable, Generator, Sequence
from xml.etree import ElementTree
from xml.etree.ElementTree import Element

logger = logging.getLogger("maltego-trx")


def name_to_path(name):
    # Convert function name to a URL path
    path = name.replace("_", "-")
    return path.lower()


def make_utf8(val):
    """
    Py2: makes variable a utf-8 encoded Unicode type
    Py3: make variable a utf-8 encoded str type
    :param val: the variable we want unicode encoded
    :return: val {Byte/Unicode}
    """
    return force_encoding(val, 'utf-8')


def make_printable(val):
    """
    Py2: makes variable an ascii encoded Unicode type
    Py3: make variable an ascii encoded str type
    :param val: the variable we want unicode encoded
    :return: val {Byte/Unicode}
    """
    return force_encoding(val, 'ascii')


def force_encoding(val, encoding):
    if isinstance(val, str):
        return val.encode(encoding, 'replace').decode(encoding, 'replace')
    elif isinstance(val, bytes):
        return val.decode(encoding, 'replace')
    else:
        return str(val).encode(encoding, 'replace').decode(encoding, 'replace')


def remove_invalid_xml_chars(val):
    """
    Remove characters which aren't allowed in XML.
    :param val:
    :return:
    """
    val = make_utf8(val)
    val = re.sub(u'[^\u0020-\uD7FF\u0009\u000A\u000D\uE000-\uFFFD\U00010000-\U0010FFFF]+', '?', val)
    return val


T = TypeVar('T')


def filter_unique(get_identifier: Callable[[T], Hashable], collection: Iterable[T]) -> Generator[T, None, None]:
    seen = set()
    for item in collection:
        identifier = get_identifier(item)

        if identifier in seen:
            continue

        seen.add(identifier)

        yield item


def chunk_list(data: Sequence[T], max_chunk_size: int) -> Generator[Sequence[T], None, None]:
    # math.ceil:
    #   number_of_chunks: decimal-places == 0 -> perfect split
    #   number_of_chunks: decimal-places  > 0 -> need one more list to keep len(chunk) <= max_chunk_size

    number_of_chunks = math.ceil(len(data) / max_chunk_size)
    chunk_size = math.ceil(len(data) / number_of_chunks)

    for idx in range(0, len(data), chunk_size):
        yield data[idx:idx + chunk_size]


def pascal_case_to_title(name: str) -> str:
    # https://stackoverflow.com/a/1176023
    name = re.sub('(.)([A-Z][a-z]+)', r'\1 \2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1 \2', name)


def escape_csv_fields(*fields: str, separator: str = ',') -> Generator[str, None, None]:
    """if a field contains the separator, it will be quoted"""
    for f in fields:
        yield f'"{f}"' if separator in f else f


def export_as_csv(header: str, lines: Sequence[str], export_file_path: str, csv_line_limit: int = -1):
    """export a file in as many files as needed to stay below the csv_line_limit (plus header)"""
    if csv_line_limit == -1 or len(lines) <= csv_line_limit:
        with open(export_file_path, "w+") as csv_file:
            csv_file.write(header + "\n")
            csv_file.writelines(map(lambda x: x + "\n", lines))

        return

    # split file to speed-up import into pTDS, iTDS
    chunks = list(chunk_list(lines, csv_line_limit))
    for idx, chunk in enumerate(chunks, 1):
        path, extension = export_file_path.rsplit(".", 1)
        chunked_config_path = f"{path}_{idx}-{len(chunks)}.{extension}"

        with open(chunked_config_path, "w+") as csv_file:
            csv_file.write(header + "\n")
            csv_file.writelines(map(lambda x: x + "\n", chunk))


def serialize_bool(boolean: bool, serialized_true: str, serialized_false: str) -> str:
    return serialized_true if boolean else serialized_false


def serialize_xml(xml: Element,
                  indent: bool = True,
                  canonicalize: bool = True,
                  short_empty_elements: bool = True) -> str:
    if indent and sys.version_info.minor >= 9:
        ElementTree.indent(xml)

    # options are needed to have same xml output for py < 3.8 and py >= 3.8
    output = ElementTree.tostring(xml, encoding='unicode', short_empty_elements=short_empty_elements)

    if canonicalize and sys.version_info.minor >= 8:
        output = ElementTree.canonicalize(output)

    return output


# https://stackoverflow.com/a/48632082
def deprecated(message="This function is deprecated."):
    def deprecated_decorator(func):
        def deprecated_func(*args, **kwargs):
            warnings.warn("{} is a deprecated function. {}".format(func.__name__, message),
                          category=DeprecationWarning,
                          stacklevel=2)
            warnings.simplefilter('default', DeprecationWarning)
            return func(*args, **kwargs)

        return deprecated_func

    return deprecated_decorator



import logging
import os
import random
import tempfile
import zipfile
from typing import NamedTuple, List

import petname
import pytest as pytest
from pytest_mock import MockerFixture

from maltego_trx.decorator_registry import (
    TransformSetting,
    TransformRegistry,
    TRANSFORMS_CSV_HEADER,
    LEGACY_TRANSFORMS_CSV_HEADER,
    SETTINGS_CSV_HEADER,
    TransformSet,
)
from maltego_trx.server import app
from maltego_trx.utils import name_to_path, serialize_bool
from tests.test_xml import _serialize_xml


@pytest.fixture
def client():
    with app.test_client() as client:
        yield client


@pytest.fixture
def registry():
    registry: TransformRegistry = TransformRegistry(
        owner="Maltego Technologies GmbH",
        author="Maltego Support",
        host_url="localhost",
        seed_ids=["demo"],
    )
    return registry


def make_transform_setting(global_setting: bool = None):
    name = petname.generate()
    setting_type = random.choice(
        ["string", "boolean", "date", "datetime", "daterange", "url", "double", "int"]
    )

    return TransformSetting(
        name=name,
        display_name=name.title(),
        setting_type=random.choice(setting_type),
        default_value=petname.generate(),
        optional=random.choice([True, False]),
        popup=random.choice([True, False]),
        global_setting=global_setting or random.choice([True, False]),
    )


def make_transform(
    registry: TransformRegistry, settings: List[TransformSetting] = None
):
    display_name = petname.generate(separator=" ")
    input_entity = petname.generate(separator=".")
    description = petname.generate(words=10, separator=" ").title() + "."
    settings = settings or [make_transform_setting(), make_transform_setting()]
    output_entities = petname.generate(3).split("-")
    disclaimer = petname.generate(words=10, separator=" ").title() + "."

    @registry.register_transform(
        display_name, input_entity, description, settings, output_entities, disclaimer
    )
    class TestClass:
        pass

    return TestClass


def test_register_transform_decorator(registry):
    test_settings = [make_transform_setting(), make_transform_setting()]

    display_name = petname.generate(separator=" ")
    input_entity = petname.generate(separator=".")
    description = petname.generate(words=10, separator=" ").title() + "."
    output_entities = petname.generate(3).split("-")
    disclaimer = petname.generate(words=10, separator=" ").title() + "."

    @registry.register_transform(
        display_name,
        input_entity,
        description,
        test_settings,
        output_entities,
        disclaimer,
    )
    class TestClass:
        pass

    path_name = name_to_path(TestClass.__name__)

    tx_meta = registry.transform_metas.get(path_name)

    assert tx_meta
    assert tx_meta.display_name == display_name
    assert tx_meta.input_entity == input_entity
    assert tx_meta.description == description
    assert tx_meta.disclaimer == disclaimer

    assert test_settings == registry.transform_settings[path_name]


class TransformCsvLine(NamedTuple):
    owner: str
    author: str
    disclaimer: str
    description: str
    version: str
    name: str
    display_name: str
    host: str
    input_entity: str
    oauth_id: str
    settings_ids: str
    seed_ids: str
    output_entities: str

class LegacyTransformCsvLine(NamedTuple):
    owner: str
    author: str
    disclaimer: str
    description: str
    version: str
    name: str
    display_name: str
    host: str
    input_entity: str
    oauth_id: str
    settings_ids: str
    seed_ids: str

class SettingCsvLine(NamedTuple):
    name: str
    setting_type: str
    display_name: str
    default: str
    optional: str
    popup: str


def test_transform_to_csv(registry: TransformRegistry):
    random_class = make_transform(registry)

    path_name = name_to_path(random_class.__name__)

    tx_meta = registry.transform_metas.get(path_name)
    tx_settings = registry.transform_settings.get(path_name, [])

    output_path = "./transforms_with_output_entities.csv"
    registry.write_transforms_config(config_path=output_path,include_output_entities=True)

    with open(output_path) as transforms_csv:
        header = next(transforms_csv)
        assert header.rstrip("\n") == TRANSFORMS_CSV_HEADER

        line = next(transforms_csv).rstrip("\n")
        data: TransformCsvLine = TransformCsvLine(*line.split(","))

        assert data.owner == registry.owner
        assert data.author == registry.author
        assert data.disclaimer == tx_meta.disclaimer
        assert data.description == tx_meta.description
        assert data.version == registry.version
        assert data.name == tx_meta.class_name
        assert data.display_name == tx_meta.display_name
        assert data.host == os.path.join(registry.host_url, "run", path_name)
        assert data.input_entity == tx_meta.input_entity
        assert data.oauth_id == registry.oauth_settings_id
        assert data.settings_ids.split(";") == [s.id for s in tx_settings]
        assert data.seed_ids.split(";") == registry.seed_ids
        assert data.output_entities.split(";") == tx_meta.output_entities

def test_transform_to_legacy_csv(registry: TransformRegistry):
    random_class = make_transform(registry)

    path_name = name_to_path(random_class.__name__)

    tx_meta = registry.transform_metas.get(path_name)
    tx_settings = registry.transform_settings.get(path_name, [])

    output_path = "./transforms_no_output_entities.csv"
    registry.write_transforms_config(config_path=output_path,include_output_entities=False)

    with open(output_path) as transforms_csv:
        header = next(transforms_csv)
        assert header.rstrip("\n") == LEGACY_TRANSFORMS_CSV_HEADER

        line = next(transforms_csv).rstrip("\n")
        data: LegacyTransformCsvLine = LegacyTransformCsvLine(*line.split(","))

        assert data.owner == registry.owner
        assert data.author == registry.author
        assert data.disclaimer == tx_meta.disclaimer
        assert data.description == tx_meta.description
        assert data.version == registry.version
        assert data.name == tx_meta.class_name
        assert data.display_name == tx_meta.display_name
        assert data.host == os.path.join(registry.host_url, "run", path_name)
        assert data.input_entity == tx_meta.input_entity
        assert data.oauth_id == registry.oauth_settings_id
        assert data.settings_ids.split(";") == [s.id for s in tx_settings]
        assert data.seed_ids.split(";") == registry.seed_ids

def test_setting_to_csv(registry: TransformRegistry):
    local_setting = make_transform_setting(global_setting=False)

    global_setting = make_transform_setting(global_setting=True)

    registry.global_settings.append(global_setting)

    @registry.register_transform("", "", "", settings=[local_setting])
    class TestClass:
        pass

    registry.write_settings_config()
    with open("./settings.csv") as settings_csv:
        header = next(settings_csv)
        assert header.rstrip("\n") == SETTINGS_CSV_HEADER

        for line, setting in zip(
            settings_csv.readlines(), [global_setting, local_setting]
        ):
            line = line.rstrip("\n")
            data: SettingCsvLine = SettingCsvLine(*line.split(","))

            assert data.name == setting.id
            assert data.setting_type == setting.setting_type
            assert data.display_name == setting.display_name
            assert data.default == setting.default_value
            assert data.optional == serialize_bool(setting.optional, "True", "False")
            assert data.popup == serialize_bool(setting.popup, "Yes", "No")


def test_write_local_mtz_emit_global_settings_warning(
    registry: TransformRegistry, caplog, snapshot
):
    registry.global_settings = [
        TransformSetting(
            name="test", display_name="test", setting_type="string", global_setting=True
        )
    ]

    with caplog.at_level(logging.WARNING):
        list(registry._create_local_mtz())

    assert caplog.messages == snapshot


def test_write_local_mtz_emit_settings_warning(
    registry: TransformRegistry, caplog, snapshot
):
    local_setting = TransformSetting(
        name="test",
        display_name="test",
        setting_type="string",
        global_setting=True,
    )

    @registry.register_transform("", "", "", settings=[local_setting])
    class TestClass:
        pass

    with caplog.at_level(logging.WARNING):
        list(registry._create_local_mtz())

    assert caplog.messages == snapshot


def test_write_local_mtz(registry: TransformRegistry, mocker: MockerFixture, snapshot):
    mocker.patch(
        "maltego_trx.mtz.create_last_sync_timestamp",
        return_value="2022-08-10 07:52:45 UTC",
    )
    mocker.patch("maltego_trx.decorator_registry.serialize_xml", _serialize_xml)

    transform_set = TransformSet(name="test", description="Test Transform Set")

    @registry.register_transform("", "", "", transform_set=transform_set)
    class TestClass:
        pass

    mtz_files = list(registry._create_local_mtz(working_dir="/home/maltego"))

    assert mtz_files == snapshot

    files = [path for path, content in mtz_files]

    assert files == snapshot


def test_write_local_mtz_file(registry: TransformRegistry, mocker: MockerFixture, snapshot):
    mocker.patch(
        "maltego_trx.mtz.create_last_sync_timestamp",
        return_value="2022-08-10 07:52:45 UTC",
    )
    mocker.patch("maltego_trx.decorator_registry.serialize_xml", _serialize_xml)

    transform_set = TransformSet(name="test", description="Test Transform Set")

    @registry.register_transform("", "", "", transform_set=transform_set)
    class TestClass:
        pass

    with tempfile.TemporaryDirectory() as working_dir:
        mtz_path = os.path.join(working_dir, "local.mtz")

        registry.write_local_mtz(mtz_path=mtz_path)

        assert os.path.exists(mtz_path), "Local mtz file not created"

        with zipfile.ZipFile(mtz_path) as local_mtz:
            assert all(
                file.file_size > 0 for file in local_mtz.infolist()
            ), "Empty files in local mtz"



from setuptools import setup
from maltego_trx import VERSION

with open("README.md", "r") as readme_md:
    long_description = readme_md.read()

setup(
    name='maltego-trx',
    python_requires='>=3.8.0',
    version=VERSION,
    description='Python library used to develop Maltego transforms',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/paterva/maltego-trx/',
    author='Maltego Staff',
    author_email='support@maltego.com',
    license='MIT',
    install_requires=[
        'flask>=2.2.0',
        'cryptography>=39.0.1'
    ],
    packages=[
        'maltego_trx',
        'maltego_trx/template_dir',
        'maltego_trx/template_dir/transforms',
    ],
    package_data={
        'maltego_trx/template_dir': [
            'settings.csv',
            'transforms.csv',
            'docker-compose.yml',
            'Dockerfile',
            'requirements.txt',
        ]
    },
    entry_points={
        'console_scripts': [
            'maltego-trx = maltego_trx.commands:execute_from_command_line',
        ]
    },
    zip_safe=False
)



import socket

from extensions import registry
from maltego_trx.entities import IPAddress
from maltego_trx.maltego import UIM_TYPES, MaltegoMsg, MaltegoTransform
from maltego_trx.transform import DiscoverableTransform


@registry.register_transform(display_name="DNS to IP", input_entity="maltego.DNSName",
                             description='Receive DNS name from the Client, and resolve to IP address.',
                             output_entities=["maltego.IPv4Address"])
class DNSToIP(DiscoverableTransform):

    @classmethod
    def create_entities(cls, request: MaltegoMsg, response: MaltegoTransform):
        dns_name = request.Value

        try:
            ip_address = socket.gethostbyname(dns_name)
            response.addEntity(IPAddress, ip_address)
        except socket.error as e:
            response.addUIMessage(f"Error: {e}", UIM_TYPES["partial"])

        # Write the slider value as a UI message - just for fun
        response.addUIMessage(f"Slider value is at: {request.Slider}")



from maltego_trx.entities import Phrase
from maltego_trx.maltego import MaltegoTransform, MaltegoMsg
from maltego_trx.template_dir.extensions import registry
from maltego_trx.template_dir.settings import language_setting

from maltego_trx.transform import DiscoverableTransform


@registry.register_transform(display_name="Greet Person (localized)", input_entity="maltego.Phrase",
                             description='Returns a localized phrase greeting a person on the graph.',
                             settings=[language_setting],
                             output_entities=["maltego.Phrase"])
class GreetPersonLocalized(DiscoverableTransform):

    @classmethod
    def create_entities(cls, request: MaltegoMsg, response: MaltegoTransform):
        person_name = request.Value

        language: str = request.getTransformSetting(language_setting.id).lower()

        if language == 'af':
            greeting = f"Hallo {person_name}, lekker om jou te ontmoet!"
        elif language == "de":
            greeting = f"Moin {person_name}, schön dich kennen zu lernen!"
        else:
            greeting = f"Hello {person_name}, nice to meet you!"

        response.addEntity(Phrase, greeting)



from extensions import registry
from maltego_trx.entities import Phrase
from maltego_trx.maltego import MaltegoMsg, MaltegoTransform

from maltego_trx.transform import DiscoverableTransform


@registry.register_transform(display_name="Greet Person", input_entity="maltego.Phrase",
                             description='Returns a Phrase greeting a Person on the Graph.',
                             output_entities=["maltego.Phrase"])
class GreetPerson(DiscoverableTransform):

    @classmethod
    def create_entities(cls, request: MaltegoMsg, response: MaltegoTransform):
        person_name = request.Value

        response.addEntity(Phrase, f"Hi {person_name}, nice to meet you!")
